/*********************************************************************
  All ADuC7023 library code provided by ADI, including this file, is provided
 as is without warranty of any kind, either expressed or implied.
 You assume any and all risk from the use of this code.  It is the
 responsibility of the person integrating this code into an application
 to ensure that the resulting application performs as required and is safe.

 Author        : ADI - Apps            www.analog.com/ADuC7023

 Date          : February 2009

 File          : I2C_Slave.c

 Hardware      : Applicable to ADuC7023 rev A later silicon
                 
 Description   : This I2C Slave example shows how to initialise the I2C peripheral in Slave mode.
 			This simple example Stores up to 6 bytes when written to by a Master.
			It will return 6 bytes when a Read request comes in from a Master
		
*********************************************************************/
#include <aduc7023.h>
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define BIT16 0x10000


unsigned char szStoreData[] = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55};	// Array to send to Slave
unsigned char ucStoreCount = 0;  		// Array index variable for szStoreData[]
unsigned int uiTest = 0;
int main(void) 
{
	POWKEY1 = 0x01;						// Configure CPU Clock for 41.78MHz, CD=0
 	POWCON = 0x00;
 	POWKEY2 = 0xF4;	

	POWKEY3 = 0x76;			 			// Enable PWM, SPI, I2C0, I2C1 Peripherals with 41.78MHz clock
    POWCON1 = 0x924;					
	POWKEY4 = 0xB1;
	// Configure P0.4 and P0.5 for I2C mode
	GP0CON |= 0x00110000;				// Only write to bits 4/5
	I2C0SCON = BIT0 		  			// Enable I2C Slave + Enable Rx interrupt +
			+ BIT9 + BIT10  			// Enable Rx interrupt + Enable Tx interrupt +
			+ BIT8;						// Enable "Stop-after-Start-Detected" IRQ.
	I2C0ID0  = 0xA0;					// Configure Slave's I2C network address
	I2C0FSTA = BIT8;					// Flush Slave Tx FIFO
	I2C0FSTA &= ~BIT8;

	uiTest = I2C0SCON;
	IRQEN = BIT9; 						// Enable I2C0 Slave interrupts

	ucStoreCount = 0;
	while (1)
	{
	}
			
}
void IRQ_Handler(void) __irq
{
	unsigned long IRQSTATUS = 0;
	unsigned int I2CSSTATUS = 0;

	IRQSTATUS = IRQSTA;	   				// Read off IRQSTA register

	if ((IRQSTATUS & BIT9) == BIT9)		//If I2C Slave interrupt source
	{
	   I2CSSTATUS = I2C0SSTA;
	   if ((I2CSSTATUS & BIT2) == BIT2) // If I2C Master Tx IRQ
	   {
			if (ucStoreCount < 6)		// Have max 6 bytes been sent?
				I2C0STX = szStoreData[ucStoreCount++];	 // Load Tx buffer
	   }
	   if ((I2CSSTATUS & BIT3) == BIT3) // If I2C Master Rx IRQ
	   {
			if (ucStoreCount < 6)		// Have max 6 bytes been received?
			{
				szStoreData[ucStoreCount] = I2C0SRX;  // Read Rx buffer
				ucStoreCount++;
			}
	   }
	   if ((I2CSSTATUS & BIT10) == BIT10)// If I2C Stop after Start condition detected IRQ
	   {
			ucStoreCount = 0;			// Reset counter Array pointer.
			I2C0FSTA = BIT8;			// Flush Slave Tx FIFO
			I2C0FSTA &= ~BIT8;
	   }
	   if ((I2CSSTATUS & BIT13) == BIT13) // If I2C Repeated Start detected IRQ
	   {
			ucStoreCount = 0;			// Reset counter Array pointer.
			I2C0FSTA = BIT8;			// Flush Slave Tx FIFO
			I2C0FSTA &= ~BIT8;
	   }
	}
}

